// SlaveMFD.cpp : Defines the exported functions for the DLL application.
//

#include "stdafx.h"


#define STRICT
#define ORBITER_MODULE
#include "Orbitersdk.h"

int MFDMode;

class VirtualMFD: public ExternMFD {
	MFDSPEC spec;
	int mode;
public:
	VirtualMFD(const MFDSPEC &spec);
	virtual void clbkRefreshDisplay(SURFHANDLE hSurf);
	virtual void clbkFocusChanged(OBJHANDLE hFocus);
	bool SetMode(int mode);
	int GetMode(void);
	bool NextMode(void);
	bool PrevMode(void);
	char *GetVesselName(void);
};

VirtualMFD::VirtualMFD(const MFDSPEC &spec) : ExternMFD(spec) 
{
	this->spec = spec;
	mode = 1;
	ExternMFD::SetMode(mode);
}

void VirtualMFD::clbkRefreshDisplay(SURFHANDLE hSurf)
{
	Log("clbkRefreshDisplay hSurf=%p", hSurf);
}

void VirtualMFD::clbkFocusChanged(OBJHANDLE hFocus) {
	Log("clbkFocusChanged %s", oapiGetVesselInterface(hFocus)->GetName());
}

int VirtualMFD::GetMode(void) {
	return mode;
}

bool VirtualMFD::SetMode(int mode) {
	if ( mode == 0 )
		return false; // Ignore request to switch MFD off

	if ( ExternMFD::SetMode(mode) ) {
		this->mode = mode;
		return true;
	}
	return false;
}

bool VirtualMFD::NextMode(void) {
	// Skip over SlaveMFD when switching modes
	int newMode = ((mode+1) == MFDMode) ? mode+2 : mode+1;
	return SetMode(newMode);
}

bool VirtualMFD::PrevMode(void) {
	// Skip over SlaveMFD when switching modes
	int newMode = ((mode-1) == MFDMode) ? mode-2 : mode-1;
	return SetMode(newMode);
}

char *VirtualMFD::GetVesselName(void) {
	return oapiGetVesselInterface(GetVessel())->GetName();
}

class SlaveMFD: public MFD {
	VirtualMFD *pMFD;
	bool committed;
	int ship;

public:
	SlaveMFD(DWORD w, DWORD h, VESSEL *vessel);
	static int MsgProc(UINT msg, UINT mfd, WPARAM wparam, LPARAM lparam);
	virtual void Update (HDC hDC);
	virtual bool  ConsumeKeyBuffered (DWORD key);
	//virtual bool  ConsumeKeyImmediate (char *kstate) { };
	virtual bool  ConsumeButton (int bt, int event);
	virtual char *ButtonLabel (int bt);
	virtual int  ButtonMenu (const MFDBUTTONMENU **menu) const;
	virtual void  WriteStatus (FILEHANDLE scn) const;
	virtual void  ReadStatus (FILEHANDLE scn);
	//virtual void  StoreStatus ();
	//virtual void  RecallStatus(); 
};

SlaveMFD::SlaveMFD(DWORD w, DWORD h, VESSEL *vessel) : MFD (w, h, vessel)
{
	Log("SlaveMFD Constructor");
	committed = false;

	ship = 0;
	MFDSPEC spec = {{0, 0, W, H}, 6, 6, H/12, H/6};
	pMFD = new VirtualMFD(spec);
	pMFD->SetVessel(oapiGetVesselByIndex(ship));
	oapiRegisterExternMFD(pMFD, spec);
}

int SlaveMFD::MsgProc(UINT msg, UINT mfd, WPARAM wparam, LPARAM lparam)
{
	Log("MsgProc msg=%d mfd=%d wparam=%d lparam=%d", msg, mfd, wparam, lparam);

	switch (msg) {
	case OAPI_MSG_MFD_OPENED:
		// Our new MFD mode has been selected, so we create the MFD and
		// return a pointer to it.
		return (int)(new SlaveMFD (LOWORD(wparam), HIWORD(wparam), (VESSEL*)lparam));
	//case OAPI_MSG_MFD_CLOSED:
		//FIXME: Delete SlaveMFD
	}
	return 0;
}


bool SlaveMFD::ConsumeKeyBuffered (DWORD key) {
	if ( committed ) {
		return pMFD->SendKey(key);
	}

	if ( key == OAPI_KEY_S ) ConsumeButton(0, PANEL_MOUSE_LBDOWN);
	if ( key == OAPI_KEY_A ) ConsumeButton(1, PANEL_MOUSE_LBDOWN);
	if ( key == OAPI_KEY_N ) ConsumeButton(2, PANEL_MOUSE_LBDOWN);
	if ( key == OAPI_KEY_B ) ConsumeButton(3, PANEL_MOUSE_LBDOWN);
	if ( key == OAPI_KEY_C ) ConsumeButton(4, PANEL_MOUSE_LBDOWN);

	return false;
}

bool  SlaveMFD::ConsumeButton (int bt, int event) {
	if ( committed ) {
		InvalidateDisplay();
		return pMFD->ProcessButton(bt, event);
	}

	if (event & PANEL_MOUSE_LBDOWN) {
		if ( bt == 0 ) {
			// Next ship
			if ( oapiGetVesselByIndex(ship+1) )
				ship = ship+1;
			pMFD->SetVessel(oapiGetVesselByIndex(ship));
			InvalidateDisplay();
			return true;
		}
		if ( bt == 1 ) {
			// Prev ship
			if ( oapiGetVesselByIndex(ship-1) )
				ship = ship-1;
			pMFD->SetVessel(oapiGetVesselByIndex(ship));
			InvalidateDisplay();
			return true;
		}
		if ( bt == 2 ) {
			if ( pMFD ) pMFD->NextMode();
			InvalidateDisplay();
			return true;
		}
		if ( bt == 3 ) {
			if ( pMFD ) pMFD->PrevMode();
			InvalidateDisplay();
			return true;
		}
		if ( bt == 4 ) {
			committed = true;
			InvalidateButtons();
			return true;
		}
	}

	return false;
}

char *SlaveMFD::ButtonLabel(int bt)
{
	Log("ButtonLabel");
	if ( committed ) 
		return strdup(pMFD->GetButtonLabel(bt));  // FIXME

	if ( bt == 0 ) return "SH+";
	if ( bt == 1 ) return "SH-";
	if ( bt == 2 ) return "MD+";
	if ( bt == 3 ) return "MD-";
	if ( bt == 4 ) return "GO";
	return NULL;
}

int SlaveMFD::ButtonMenu (const MFDBUTTONMENU **menu) const
{
	Log("ButtonMenu");

    static const MFDBUTTONMENU mnu[5] = {
        {"Next ship", 0, 'S'},
		{"Prev ship", 0, 'A'},
		{"Next MFD mode", 0, 'N'},
		{"Prev MFD mode", 0, 'B'},
		{"Commit", 0, 'C'},
    };
    if (menu) *menu = mnu;
    return 5; // return the number of buttons used
}

void SlaveMFD::Update (HDC myDC)
{
	Log("Update");

	// Copy screen
	HDC hisDC = oapiGetDC(pMFD->GetDisplaySurface());

	if (BitBlt(myDC, 0, 0, W, H, hisDC, 0, 0, SRCCOPY) == 0) {
		DWORD dwErr = GetLastError();
		Log("Update BitBlt() failed. GetLastError()=%d (0x%08x)", dwErr, dwErr);
	}
	oapiReleaseDC(pMFD->GetDisplaySurface(), hisDC);

	RECT pos = { 0, 0, W, H };
	DrawTextA(myDC, pMFD->GetVesselName(), -1, &pos, DT_SINGLELINE|DT_BOTTOM|DT_RIGHT);
	
}

void SlaveMFD::WriteStatus(FILEHANDLE scn) const
{
	char buf[1024];
	sprintf(buf, "VESSEL %s", pMFD->GetVesselName()); 
	oapiWriteLine(scn, buf);
	sprintf(buf, "MODE %d", pMFD->GetMode()); 
	oapiWriteLine(scn, buf);
}

void SlaveMFD::ReadStatus(FILEHANDLE scn)
{
	char *line;
	while (oapiReadScenario_nextline (scn, line)) {
		if (_strnicmp (line, "VESSEL", 6) == 0) {
			if ( oapiGetObjectByName(line+7) ) {
				pMFD->SetVessel(oapiGetObjectByName(line+7));

				// Find vessel index by name
				for (int i=0; i<oapiGetVesselCount(); i++) {
					VESSEL *v = oapiGetVesselInterface(oapiGetVesselByIndex(i));
					char *name = v->GetName();
					if ( stricmp(line+7, name) == 0 ) {
						ship = i;
						break;
					}
				}
			}
		}
		else if (_strnicmp (line, "MODE", 4) == 0) {
			pMFD->SetMode(atoi(line+5));
		}
	}
}

// *************** Module stuff *****************


HINSTANCE hDllInstance;

class SlaveMFDModule : oapi::Module {
public:
	SlaveMFDModule(HINSTANCE hDLL) : oapi::Module(hDLL) { };
	virtual void clbkPostStep(double simt, double simdt, double mjd);
};

void SlaveMFDModule::clbkPostStep(double simt, double simdt, double mjd)
{

}

DLLCLBK void InitModule (HINSTANCE hDLL)
{
	initLog("SlaveMFD.log");
	Log("DLL loaded, build: %s %s", __DATE__, __TIME__);

	hDllInstance = hDLL; // remember the instance handle

	SlaveMFDModule *mod = new SlaveMFDModule(hDllInstance);
	oapiRegisterModule((oapi::Module*) mod);


	static char *name = "SlaveMFD";
	MFDMODESPECEX spec;
	spec.name    = name;
	spec.key     = OAPI_KEY_S;
	spec.context = NULL;
	spec.msgproc = SlaveMFD::MsgProc;

	MFDMode = oapiRegisterMFDMode(spec);
	Log("MFDMode=%d", MFDMode);
}



DLLCLBK void ExitModule (HINSTANCE hDLL)
{
	Log("ExitModule()");
	closeLog();
}

